@@ -22,14 +22,14 @@
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
  *
  */
-/** 
+/**
  * SECTION:ctk-box
  * @short_description: Base class for box containers
  * @see_also: #CtkVBox #CtkHBox
  * @include ctk-box.h
  *
  * #CtkBox is a base class for our box containers, It handles all the logic for
- * #CtkVbox and #CtkHbox 
+ * #CtkVbox and #CtkHbox
  */
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -63,7 +63,7 @@ struct _CtkBoxPrivate
   gboolean            spacing_set;
 
   ClutterActor *child;
-  
+
   GCompareFunc compare_func;
 };
 
@@ -154,17 +154,39 @@ clutter_container_iface_init (ClutterContainerIface *iface)
 static void
 ctk_box_paint (ClutterActor *actor)
 {
-  GList *c;
+  CtkActor *ctk_actor = CTK_ACTOR (actor);
+  GSList   *effects   = NULL;
 
-  CLUTTER_ACTOR_CLASS (ctk_box_parent_class)->paint (actor);
-
-  for (c = CTK_BOX (actor)->priv->children; c; c = c->next)
+  if (!ctk_actor_get_effects_painting (ctk_actor)
+      && (effects = ctk_actor_get_effects (ctk_actor)))
     {
-      CtkBoxChild *child_info = c->data;
+      GSList *e;
 
-      if (CLUTTER_ACTOR_IS_VISIBLE (child_info->actor))
-        clutter_actor_paint (child_info->actor);
+      ctk_actor_set_effects_painting (ctk_actor, TRUE);
+
+      for (e = effects; e; e = e->next)
+        {
+          gboolean last_effect = e->next ? FALSE : TRUE;
+          ctk_effect_paint (e->data, ctk_box_paint, last_effect);
+        }
+
+      ctk_actor_set_effects_painting (ctk_actor, FALSE);
     }
+  else
+    {
+      GList *c;
+
+      CLUTTER_ACTOR_CLASS (ctk_box_parent_class)->paint (actor);
+
+      for (c = CTK_BOX (actor)->priv->children; c; c = c->next)
+        {
+          CtkBoxChild *child_info = c->data;
+
+          if (CLUTTER_ACTOR_IS_VISIBLE (child_info->actor))
+            clutter_actor_paint (child_info->actor);
+        }
+    }
+  ctk_actor_set_damaged (ctk_actor, FALSE);
 }
 
 static void
@@ -264,6 +286,9 @@ ctk_box_get_preferred_width (ClutterActor *actor,
           child = children->data;
           children = children->next;
 
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+            continue;
+
           clutter_actor_get_preferred_width (child->actor, for_height,
                                              &cmin_width, &cnat_width);
 
@@ -388,6 +413,9 @@ ctk_box_get_preferred_height (ClutterActor *actor,
           child = c->data;
           gfloat height;
 
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+            continue;
+
           clutter_actor_get_preferred_height (child->actor,
                                               for_width, NULL, &height);
 
@@ -441,7 +469,7 @@ ctk_box_get_preferred_height (ClutterActor *actor,
               if (natural_height)
                 *natural_height = MAX (*natural_height, nheight);
             }
-          else
+          else if (priv->orientation == CTK_ORIENTATION_VERTICAL)
             {
               if (minimum_height)
                 *minimum_height += min_height;
@@ -514,6 +542,10 @@ ctk_box_allocate (ClutterActor          *actor,
   for (c = priv->children; c; c = c->next)
     {
       CtkBoxChild *child = c->data;
+
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+        continue;
+
       n_children++;
 
       if (child->expand || priv->homogeneous)
@@ -531,6 +563,9 @@ ctk_box_allocate (ClutterActor          *actor,
           gfloat min_width;
           gfloat min_height;
 
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+              continue;
+
           if (child->expand)
             continue;
 
@@ -571,6 +606,9 @@ ctk_box_allocate (ClutterActor          *actor,
           gfloat       min_width;
           gfloat       min_height;
 
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+              continue;
+
           if (!child->expand)
             continue;
 
@@ -611,6 +649,9 @@ ctk_box_allocate (ClutterActor          *actor,
       gfloat       child_width;
       gfloat       child_height;
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+          continue;
+
       clutter_actor_get_allocation_box (child->actor, &child_box);
 
       if (child->expand || priv->homogeneous)
@@ -674,7 +715,11 @@ ctk_box_pick (ClutterActor       *actor,
    * the child actors also getting 'picked'.
    */
   if (CLUTTER_ACTOR_IS_VISIBLE (actor))
-    ctk_box_paint (actor);
+    {
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), TRUE);
+      ctk_box_paint (actor);
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), FALSE);
+    }
 }
 
 static void
@@ -857,6 +902,9 @@ ctk_box_focus_in_event  (ClutterActor    *actor)
     {
       CtkBoxChild *child = c->data;
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+          continue;
+
       if (CTK_IS_FOCUSABLE (child->actor))
         {
           ClutterActorBox box;
@@ -901,6 +949,9 @@ ctk_box_key_press_event (ClutterActor    *actor,
       CtkBoxChild  *box_child = c->data;
       ClutterActor *child = box_child->actor;
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+          continue;
+
       if (CTK_IS_FOCUSABLE (child)
           && ctk_focusable_get_focused (CTK_FOCUSABLE (child)))
         {
@@ -959,6 +1010,10 @@ ctk_box_key_press_event (ClutterActor    *actor,
       for (c = g_list_nth (priv->children, focused_child_pos+1); c; c = c->next)
         {
           CtkBoxChild *child = c->data;
+
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+            continue;
+
           if (CTK_IS_FOCUSABLE (child->actor))
             {
               next_focus = child->actor;
@@ -988,6 +1043,10 @@ ctk_box_key_press_event (ClutterActor    *actor,
       for (c = g_list_nth (priv->children, focused_child_pos-1); c; c = c->prev)
         {
           CtkBoxChild *child = c->data;
+
+          if (!CLUTTER_ACTOR_IS_VISIBLE (child->actor))
+            continue;
+
           if (CTK_IS_FOCUSABLE (child->actor))
             {
               next_focus = child->actor;
@@ -1014,9 +1073,9 @@ ctk_box_key_press_event (ClutterActor    *actor,
  * Public methods
  */
 
-/** 
+/**
  * ctk_box_pack:
- * @box: A CtkBox 
+ * @box: A CtkBox
  * @child: The ClutterActor that is to be added to the @box
  * @expand: TRUE if the @actor is to be given extra space allocated to @box
  * @fill: TRUE if space given to @actor by the expand option is actually allocated to @actor, rather than just padding it.
@@ -1082,12 +1141,12 @@ ctk_box_set_homogeneous (CtkBox  *box,
     }
 }
 
-/** 
+/**
  * ctk_box_get_homogeneous:
  * @box: a CtkBox
  *
  * gets whether the @box is homogeneous, see ctk_box_set_homogeneous()
- * Returns: TRUE or FALSE 
+ * Returns: TRUE or FALSE
  */
 gboolean
 ctk_box_get_homogeneous (CtkBox  *box)
@@ -1096,7 +1155,7 @@ ctk_box_get_homogeneous (CtkBox  *box)
   return box->priv->homogeneous;
 }
 
-/** 
+/**
  * ctk_box_set_spacing:
  * @box: a CtkBox
  * @spacing: a gint refering to the size of the spacing in pixels
@@ -1120,13 +1179,13 @@ ctk_box_set_spacing (CtkBox *box,
     }
 }
 
-/** 
+/**
  * ctk_box_get_spacing:
  * @box: a CtkBox
  *
  * Returns the spacing of @box, see also ctk_box_set_spacing()
  *
- * Returns: An integer regarding @box spacing 
+ * Returns: An integer regarding @box spacing
  */
 gint
 ctk_box_get_spacing (CtkBox *box)
@@ -1135,7 +1194,7 @@ ctk_box_get_spacing (CtkBox *box)
   return box->priv->spacing;
 }
 
-/** 
+/**
  * ctk_box_set_orientation:
  * @box: a CtkBox
  * @orient: a CtkOrientation orientation
@@ -1154,7 +1213,7 @@ ctk_box_set_orientation (CtkBox *box,
   g_object_notify (G_OBJECT (box), "orientation");
 }
 
-/** 
+/**
  * ctk_box_get_orientation:
  * @box: a CtkBox
  *
@@ -1179,7 +1238,7 @@ sort_func (CtkBoxChild *achild, CtkBoxChild *bchild, CtkBox *box)
   return priv->compare_func (achild->actor, bchild->actor);
 }
 
-/** 
+/**
  * ctk_box_sort_children:
  * @box: a CtkBox
  * @func: a GCompareFunc to be used to sort children in @box
