@@ -16,13 +16,13 @@
  * Authored by Neil Jagdish Patel <neil.patel@canonical.com>
  *
  */
-/** 
+/**
  * SECTION:ctk-button
  * @short_description: A widget that presents a button interface
  * @include ctk-button.h
  *
- * #CtkButton is used to present a classical "button" interface and to provide 
- * appropriate signals when it is clicked on 
+ * #CtkButton is used to present a classical "button" interface and to provide
+ * appropriate signals when it is clicked on
  * Typically a CtkButton will/can hold a #CtkText and #CtkImage at the same time.
  */
 
@@ -202,7 +202,7 @@ get_preferred_width (ClutterActor *actor,
                                          NULL, &nwidth);
 
       nwidth += padding.left + padding.right;
-      nwidth += 12.0;
+      nwidth += 6.0;
 
       clutter_text_set_line_wrap (CLUTTER_TEXT (priv->text), FALSE);
       layout = clutter_text_get_layout (CLUTTER_TEXT (priv->text));
@@ -282,7 +282,7 @@ get_preferred_height (ClutterActor *actor,
                                           NULL, &nheight);
 
       nheight += padding.top + padding.right;
-      nheight += 12.0;
+      nheight += 6.0;
 
       layout = clutter_text_get_layout (CLUTTER_TEXT (priv->text));
       pango_layout_set_width (layout,
@@ -325,15 +325,12 @@ allocate (ClutterActor          *actor,
     {
       ClutterActorBox child_box;
 
-      child_box.x1 = padding.left;
-      child_box.x2 = padding.left + real_width;
-      child_box.y1 = padding.top;
-      child_box.y2 = padding.top + real_height;
+      child_box.x1 = floor (padding.left);
+      child_box.x2 = floor (padding.left + real_width);
+      child_box.y1 = floor (padding.top);
+      child_box.y2 = floor (padding.top + real_height);
 
       clutter_actor_allocate (child, &child_box, flags);
-
-      clutter_actor_hide (priv->image);
-      clutter_actor_hide (priv->text);
       return;
     }
 
@@ -350,17 +347,17 @@ allocate (ClutterActor          *actor,
 
           clutter_actor_get_preferred_width (priv->image, real_height,
                                              NULL, &nat_width);
-          child_box.x1 = x;
-          child_box.x2 = x + nat_width;
-          child_box.y1 = y;
-          child_box.y2 = y + real_height;
+          child_box.x1 = floor (x);
+          child_box.x2 = floor (x + nat_width);
+          child_box.y1 = floor (y);
+          child_box.y2 = floor (y + real_height);
 
           if (direction == GTK_TEXT_DIR_RTL)
             {
-              child_box.x1 = padding.left + real_width - nat_width;
-              child_box.x2 = child_box.x1 + nat_width;
+              child_box.x1 = floor (padding.left + real_width - nat_width);
+              child_box.x2 = floor (child_box.x1 + nat_width);
             }
-          x += nat_width + 12.0;
+          x += nat_width + 6.0;
         }
       else
         {
@@ -368,12 +365,12 @@ allocate (ClutterActor          *actor,
 
           clutter_actor_get_preferred_height (priv->image, real_width,
                                               NULL, &nat_height);
-          child_box.x1 = x;
-          child_box.x2 = x + real_width;
-          child_box.y1 = y;
-          child_box.y2 = y + nat_height;
+          child_box.x1 = floor (x);
+          child_box.x2 = floor (x + real_width);
+          child_box.y1 = floor (y);
+          child_box.y2 = floor (y + nat_height);
 
-          y += nat_height + 12.0;
+          y += nat_height + 6.0;
         }
       clutter_actor_allocate (priv->image, &child_box, flags);
     }
@@ -404,20 +401,10 @@ allocate (ClutterActor          *actor,
         }
       else
         {
-          gfloat text_width = GFLOAT_FROM_PANGO_UNIT (log_rect.width);
-          gint lines = pango_layout_get_line_count (layout);
-
-          child_box.x1 = x;
-          child_box.x2 = x + (real_width );
-          child_box.y1 = y;
-          child_box.y2 = y + (real_height - y);
-
-          if (lines == 1)
-            {
-              child_box.x1 = x + ((real_width)/2) - (text_width/2);
-              child_box.x2 = GFLOAT_FROM_PANGO_UNIT (text_width);
-            }
-
+          child_box.x1 = floor (x);
+          child_box.x2 = floor (x + (real_width ));
+          child_box.y1 = floor (y);
+          child_box.y2 = floor (y + (real_height - y));
         }
       clutter_actor_allocate (priv->text, &child_box, flags);
       clutter_actor_show (priv->text);
@@ -427,22 +414,44 @@ allocate (ClutterActor          *actor,
 static void
 paint (ClutterActor *actor)
 {
-  CtkButtonPrivate *priv = CTK_BUTTON (actor)->priv;
-  ClutterActor *child;
+  CtkActor *ctk_actor = CTK_ACTOR (actor);
+  GSList   *effects   = NULL;
 
-  CLUTTER_ACTOR_CLASS (ctk_button_parent_class)->paint (actor);
-
-  child = ctk_bin_get_child (CTK_BIN (actor));
-  if (child)
+  if (!ctk_actor_get_effects_painting (ctk_actor)
+      && (effects = ctk_actor_get_effects (ctk_actor)))
     {
-      clutter_actor_paint (child);
-      return;
+      GSList *e;
+
+      ctk_actor_set_effects_painting (ctk_actor, TRUE);
+
+      for (e = effects; e; e = e->next)
+        {
+          gboolean last_effect = e->next ? FALSE : TRUE;
+          ctk_effect_paint (e->data, paint, last_effect);
+        }
+
+      ctk_actor_set_effects_painting (ctk_actor, FALSE);
     }
   else
     {
-      clutter_actor_paint (priv->image);
-      clutter_actor_paint (priv->text);
+      CtkButtonPrivate *priv = CTK_BUTTON (actor)->priv;
+      ClutterActor *child;
+
+      CLUTTER_ACTOR_CLASS (ctk_button_parent_class)->paint (actor);
+
+      child = ctk_bin_get_child (CTK_BIN (actor));
+      if (child)
+        {
+          clutter_actor_paint (child);
+          return;
+        }
+      else
+        {
+          clutter_actor_paint (priv->image);
+          clutter_actor_paint (priv->text);
+        }
     }
+  ctk_actor_set_damaged (ctk_actor, FALSE);
 }
 
 static void
@@ -554,7 +563,7 @@ ctk_button_init (CtkButton *button)
 
   priv->text = ctk_text_new (NULL);
   clutter_text_set_line_wrap (CLUTTER_TEXT (priv->text), TRUE);
-  clutter_text_set_line_wrap_mode (CLUTTER_TEXT (priv->text), 
+  clutter_text_set_line_wrap_mode (CLUTTER_TEXT (priv->text),
                                    PANGO_WRAP_WORD_CHAR);
   clutter_actor_set_parent (priv->text, CLUTTER_ACTOR (button));
   clutter_actor_show (priv->text);
@@ -572,7 +581,7 @@ ctk_button_init (CtkButton *button)
  *
  * Creates a new CtkButton object
  *
- * returns: A new CtkButton object 
+ * returns: A new CtkButton object
  */
 ClutterActor *
 ctk_button_new (CtkOrientation orientation)
@@ -625,6 +634,8 @@ on_button_press (ClutterActor       *actor,
   if (event->button == 1)
     {
       priv->last_press_time = event->time;
+
+      return TRUE;
     }
   else if (event->button == 3)
     {
@@ -650,7 +661,7 @@ on_enter (ClutterActor         *actor,
 
   ctk_actor_set_state (CTK_ACTOR (actor), CTK_STATE_PRELIGHT);
 
-  return FALSE;
+  return TRUE;
 }
 
 static gboolean
@@ -668,7 +679,7 @@ on_leave (ClutterActor         *actor,
 
   ctk_actor_set_state (CTK_ACTOR (actor), CTK_STATE_NORMAL);
 
-  return FALSE;
+  return TRUE;
 }
 
 static void
@@ -757,7 +768,7 @@ ctk_button_set_image (CtkButton  *button,
 /**
  * ctk_button_get_image:
  * @button: A CtkButton
- * 
+ *
  * returns the GtkImage that is attached to @button. see also: ctk_button_set_image()
  *
  * Returns: a CtkImage;
