@@ -22,7 +22,7 @@
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
  *
  */
-/** 
+/**
  * SECTION:ctk-icon-view
  * @short_description: A widget that organises its children into a grid of icons
  * @include: ctk-icon-view.h
@@ -59,13 +59,21 @@ struct _CtkIconViewPrivate
   gint16  spacing;
 
   guint   n_cols;
+  guint   n_rows;
+
+  gfloat last_height;
+
+  gboolean auto_fade_children;
 };
 
 enum
 {
   PROP_0,
 
-  PROP_SPACING
+  PROP_SPACING,
+  PROP_AUTO_FADE_CHILDREN,
+  PROP_N_COLS,
+  PROP_N_ROWS
 };
 
 /* Globals */
@@ -101,7 +109,7 @@ ctk_icon_view_finalize (GObject *object)
         clutter_actor_unparent (child);
     }
   g_list_free (priv->children);
-  priv->children = NULL;  
+  priv->children = NULL;
 
   G_OBJECT_CLASS (ctk_icon_view_parent_class)->finalize (object);
 }
@@ -120,22 +128,48 @@ clutter_container_iface_init (ClutterContainerIface *iface)
 static void
 ctk_icon_view_paint (ClutterActor *actor)
 {
-  GList *c;
+  CtkActor *ctk_actor = CTK_ACTOR (actor);
+  GSList   *effects   = NULL;
 
-  CLUTTER_ACTOR_CLASS (ctk_icon_view_parent_class)->paint (actor);
-
-
-  for (c = CTK_ICON_VIEW (actor)->priv->children; c; c = c->next)
+  if (!ctk_actor_get_effects_painting (ctk_actor)
+      && (effects = ctk_actor_get_effects (ctk_actor)))
     {
-      if (CLUTTER_ACTOR_IS_VISIBLE (c->data))
-        clutter_actor_paint (c->data);
+      GSList *e;
+
+      ctk_actor_set_effects_painting (ctk_actor, TRUE);
+
+      for (e = effects; e; e = e->next)
+        {
+          gboolean last_effect = e->next ? FALSE : TRUE;
+          ctk_effect_paint (e->data, ctk_icon_view_paint, last_effect);
+        }
+
+      ctk_actor_set_effects_painting (ctk_actor, FALSE);
     }
+  else
+    {
+      GList *c;
+
+      CLUTTER_ACTOR_CLASS (ctk_icon_view_parent_class)->paint (actor);
+
+      for (c = CTK_ICON_VIEW (actor)->priv->children; c; c = c->next)
+        {
+          if (CLUTTER_IS_ACTOR (c->data) &&
+              CLUTTER_ACTOR_IS_VISIBLE (c->data) &&
+              clutter_actor_get_opacity (c->data) != 0)
+            {
+              clutter_actor_paint (c->data);
+            }
+        }
+    }
+  ctk_actor_set_damaged (ctk_actor, FALSE);
 }
 
 static void
 ctk_icon_view_pick (ClutterActor       *actor,
                     const ClutterColor *color)
 {
+
   /* Chain up so we get a bounding icon_view pained (if we are reactive) */
   CLUTTER_ACTOR_CLASS (ctk_icon_view_parent_class)->pick (actor, color);
 
@@ -143,7 +177,11 @@ ctk_icon_view_pick (ClutterActor       *actor,
    * the child actors also getting 'picked'.
    */
   if (CLUTTER_ACTOR_IS_VISIBLE (actor))
-    ctk_icon_view_paint (actor);
+    {
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), TRUE);
+      ctk_icon_view_paint (actor);
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), FALSE);
+    }
 }
 
 static void
@@ -159,6 +197,9 @@ ctk_icon_view_set_property (GObject      *object,
     case PROP_SPACING:
       ctk_icon_view_set_spacing (icon_view, g_value_get_int (value));
       break;
+    case PROP_AUTO_FADE_CHILDREN:
+      icon_view->priv->auto_fade_children = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -178,6 +219,15 @@ ctk_icon_view_get_property (GObject    *object,
     case PROP_SPACING:
       g_value_set_int (value, ctk_icon_view_get_spacing (icon_view));
       break;
+    case PROP_AUTO_FADE_CHILDREN:
+      g_value_set_boolean (value, icon_view->priv->auto_fade_children);
+      break;
+    case PROP_N_ROWS:
+      g_value_set_uint (value, icon_view->priv->n_rows);
+      break;
+    case PROP_N_COLS:
+      g_value_set_uint (value, icon_view->priv->n_cols);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -242,6 +292,9 @@ ctk_icon_view_get_preferred_height (ClutterActor *actor,
       ClutterActor *child = c->data;
       gfloat   nwidth = 0;
 
+      if (!CLUTTER_IS_ACTOR (child) || !CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       clutter_actor_get_preferred_width (child, -1, NULL, &nwidth);
 
       max_child_width = MAX (max_child_width, nwidth);
@@ -261,6 +314,9 @@ ctk_icon_view_get_preferred_height (ClutterActor *actor,
       ClutterActor    *child = c->data;
       gfloat      nheight = 0;
 
+      if (!CLUTTER_IS_ACTOR (child) || !CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       clutter_actor_get_preferred_height (child, max_child_width,NULL, &nheight);
 
       cur_row_height = MAX (cur_row_height, nheight);
@@ -316,12 +372,17 @@ ctk_icon_view_allocate (ClutterActor          *actor,
   real_width = box->x2 - box->x1 - padding.left - padding.right;
   spacing = (gfloat) (priv->spacing);
 
+  priv->last_height = box->y2 - box->y1;
+
   /* Find out our widest child */
   for (c = priv->children; c; c = c->next)
     {
       ClutterActor *child = c->data;
       gfloat   nwidth = 0;
 
+      if (!CLUTTER_IS_ACTOR (child) || !CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       clutter_actor_get_preferred_width (child, -1, NULL, &nwidth);
 
       max_child_width = MAX (max_child_width, nwidth);
@@ -331,18 +392,27 @@ ctk_icon_view_allocate (ClutterActor          *actor,
   /* Figure out the layout */
   n_cols = floor (real_width/ max_child_width);
   n_cols = (real_width - ((n_cols-1) * spacing))/max_child_width;
-  priv->n_cols = n_cols;
+
+  if (priv->n_cols != n_cols)
+    {
+      priv->n_cols = n_cols;
+      g_object_notify (G_OBJECT (actor), "n-cols");
+    }
 
   x = padding.left;
   y = padding.top;
 
   gfloat row_heights[(gint)ceil (i/n_cols)+1];
-  
+  row_heights[0] = 0.0f;
+
   i = 0;
   for (c = priv->children; c; c = c->next)
     {
       gfloat nheight =0;
 
+      if (!CLUTTER_IS_ACTOR (c->data) || !CLUTTER_ACTOR_IS_VISIBLE (c->data))
+        continue;
+
       clutter_actor_get_preferred_height (c->data, max_child_width, NULL,
                                           &nheight);
 
@@ -352,8 +422,15 @@ ctk_icon_view_allocate (ClutterActor          *actor,
         {
           row += 1;
           i = 0;
+          row_heights[row] = 0.0f;
         }
     }
+  row += (i == 0 ? 0 : 1);
+  if (priv->n_rows != row)
+    {
+      priv->n_rows = row;
+      g_object_notify (G_OBJECT (actor), "n-rows");
+    }
 
   i = 0;
   row = 0;
@@ -362,26 +439,46 @@ ctk_icon_view_allocate (ClutterActor          *actor,
     {
       ClutterActor    *child = c->data;
       ClutterActorBox  child_box;
-      ClutterActorBox *box;
+      ClutterActorBox *box_;
 
-      box = g_object_get_data (G_OBJECT (child), "ctk-icon-view-child-box");
+      if (!CLUTTER_IS_ACTOR (child) || !CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
 
-      child_box.x1 = x;
-      child_box.x2 = x + max_child_width;
-      child_box.y1 = y;
-      child_box.y2 = y + row_heights[row];
+      box_ = g_object_get_data (G_OBJECT (child), "ctk-icon-view-child-box");
+
+      child_box.x1 = floor (x);
+      child_box.x2 = floor (x + max_child_width);
+      child_box.y1 = floor (y);
+      child_box.y2 = floor (y + row_heights[row]);
       clutter_actor_allocate (child, &child_box, flags);
 
-      if (box)
+      if (box_)
         {
-          box->x1 = child_box.x1; box->x2 = child_box.x2;
-          box->y1 = child_box.y1; box->y2 = child_box.y2;
+          box_->x1 = floor (child_box.x1); box_->x2 = floor (child_box.x2);
+          box_->y1 = floor (child_box.y1); box_->y2 = floor (child_box.y2);
         }
 
       cur_row_height = MAX (cur_row_height, row_heights[row]);
       x += max_child_width + spacing;
       i += 1;
 
+      if (priv->auto_fade_children)
+        {
+          if (child_box.y2 <= priv->last_height + padding.top)
+            clutter_actor_set_opacity (child, 255);
+
+          else if (child_box.y1 > priv->last_height)
+            {
+              clutter_actor_set_opacity (child, 0);
+            }
+          else if (child_box.y1 < priv->last_height &&
+                   child_box.y2 > priv->last_height)
+            {
+              clutter_actor_set_opacity (child,
+                                         255 * ((priv->last_height - child_box.y2)/child_box.y1));
+            }
+        }
+
       if (i == n_cols)
         {
           x = padding.left;
@@ -408,6 +505,9 @@ ctk_icon_view_focus_in (ClutterActor *actor)
     {
       ClutterActor *child = c->data;
 
+      if (!CLUTTER_IS_ACTOR (child) || !CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       if (CTK_IS_FOCUSABLE (child))
         {
           ClutterActorBox *box;
@@ -474,6 +574,9 @@ ctk_icon_view_key_press_event (ClutterActor *actor, ClutterKeyEvent *event)
     {
       ClutterActor *child = c->data;
 
+      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
+        continue;
+
       if (CTK_IS_FOCUSABLE (child)
           && ctk_focusable_get_focused (CTK_FOCUSABLE (child)))
         {
@@ -544,6 +647,26 @@ ctk_icon_view_class_init (CtkIconViewClass *klass)
                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
   g_object_class_install_property (obj_class, PROP_SPACING, pspec);
 
+  pspec = g_param_spec_boolean ("auto-fade-children",
+                                "Auto Fade Children",
+                                "Fade out children that are not within the "
+                                "bounds of the iconview",
+                                FALSE,
+                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+  g_object_class_install_property (obj_class, PROP_AUTO_FADE_CHILDREN, pspec);
+
+  pspec = g_param_spec_uint ("n-rows", "n-rows",
+                             "Number of rows in the iconview",
+                             0, G_MAXINT, 0,
+                             G_PARAM_READABLE);
+  g_object_class_install_property (obj_class, PROP_SPACING, pspec);
+
+  pspec = g_param_spec_uint ("n-cols", "n-cols",
+                             "Number of columns in the iconview",
+                             0, G_MAXINT, 0,
+                             G_PARAM_READABLE);
+  g_object_class_install_property (obj_class, PROP_SPACING, pspec);
+
   /* Install private struct */
   g_type_class_add_private (obj_class, sizeof (CtkIconViewPrivate));
 }
@@ -557,6 +680,8 @@ ctk_icon_view_init (CtkIconView *icon_view)
 
   priv->children = NULL;
   priv->spacing = 0;
+  priv->last_height = 0.0f;
+  priv->auto_fade_children = FALSE;
 }
 
 ClutterActor *
@@ -664,7 +789,8 @@ ctk_icon_view_foreach (ClutterContainer *container,
 
   for (c = priv->children; c; c = c->next)
     {
-      (* callback) (c->data, user_data);
+      if (CLUTTER_IS_ACTOR (c->data))
+        (* callback) (c->data, user_data);
     }
 }
 
@@ -720,10 +846,10 @@ ctk_icon_view_set_spacing (CtkIconView *icon_view,
 /**
  * ctk_icon_view_get_spacing:
  * @icon_view: A CtkIconView
- * 
- * Gets the spacing defined previously in ctk_icon_view_set_spacing() 
  *
- * Returns: An integer defining the spacing in pixels 
+ * Gets the spacing defined previously in ctk_icon_view_set_spacing()
+ *
+ * Returns: An integer defining the spacing in pixels
  */
 
 gint
@@ -732,3 +858,29 @@ ctk_icon_view_get_spacing (CtkIconView *icon_view)
   g_return_val_if_fail (CTK_IS_ICON_VIEW (icon_view), 0);
   return icon_view->priv->spacing;
 }
+
+/**
+ * ctk_icon_view_get_n_cols:
+ * @icon_view: a #CtkIconView
+ *
+ * Returns: The number of columns in the @icon_view
+ */
+guint
+ctk_icon_view_get_n_cols (CtkIconView  *icon_view)
+{
+  g_return_val_if_fail (CTK_IS_ICON_VIEW (icon_view), 0);
+  return icon_view->priv->n_cols;
+}
+
+/**
+ * ctk_icon_view_get_n_rows:
+ * @icon_view: a #CtkIconView
+ *
+ * Returns: The number of rows in the @icon_view
+ */
+guint
+ctk_icon_view_get_n_rows (CtkIconView  *icon_view)
+{
+  g_return_val_if_fail (CTK_IS_ICON_VIEW (icon_view), 0);
+  return icon_view->priv->n_rows;
+}
