@@ -1,3 +1,4 @@
+/* -*- Mode: c; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
 /*
  * Copyright 2009 Canonical Ltd.
  *
@@ -22,12 +23,12 @@
  * Authored by: Neil Jagdish Patel <neil.patel@canonical.com>
  *
  */
-/** 
+/**
  * SECTION:ctk-image
  * @short_description: A Widget that presents graphical texture data
  * @include: ctk-button.h
  *
- * #CtkImage is designed to present graphical images whist being able to load 
+ * #CtkImage is designed to present graphical images whist being able to load
  * from intelligent sources such as stock icons and icon names
  */
 
@@ -35,6 +36,8 @@
 #include <config.h>
 #endif
 
+#include <math.h>
+
 #include "ctk-image.h"
 
 G_DEFINE_TYPE (CtkImage, ctk_image, CTK_TYPE_ACTOR);
@@ -75,6 +78,7 @@ enum
 /* Globals */
 
 /* Forwards */
+static void free_resources (CtkImage *image);
 
 /* GObject stuff */
 static void
@@ -88,6 +92,8 @@ finalize (GObject *object)
       priv->texture = NULL;
     }
 
+  free_resources (CTK_IMAGE (object));
+
   G_OBJECT_CLASS (ctk_image_parent_class)->finalize (object);
 }
 
@@ -219,10 +225,10 @@ allocate (ClutterActor          *actor,
   width = box->x2 - box->x1 - padding.left - padding.right;
   height = box->y2 - box->y1 - padding.top - padding.bottom;
 
-  child_box.x1 = padding.left + ((width/2) - (img_width/2));
-  child_box.x2 = child_box.x1 + img_width;
-  child_box.y1 = padding.top + ((height/2) - (img_height/2));
-  child_box.y2 = child_box.y1 + img_height;
+  child_box.x1 = floor (padding.left + ((width/2) - (img_width/2)));
+  child_box.x2 = floor (child_box.x1 + img_width);
+  child_box.y1 = floor (padding.top + ((height/2) - (img_height/2)));
+  child_box.y2 = floor (child_box.y1 + img_height);
 
   clutter_actor_allocate (CTK_IMAGE (actor)->priv->texture,
                           &child_box, flags);
@@ -231,9 +237,31 @@ allocate (ClutterActor          *actor,
 static void
 paint (ClutterActor *actor)
 {
-  CLUTTER_ACTOR_CLASS (ctk_image_parent_class)->paint (actor);
+  CtkActor *ctk_actor = CTK_ACTOR (actor);
+  GSList   *effects   = NULL;
 
-  clutter_actor_paint (CTK_IMAGE (actor)->priv->texture);
+  if (!ctk_actor_get_effects_painting (ctk_actor)
+      && (effects = ctk_actor_get_effects (ctk_actor)))
+    {
+      GSList *e;
+
+      ctk_actor_set_effects_painting (ctk_actor, TRUE);
+
+      for (e = effects; e; e = e->next)
+        {
+          gboolean last_effect = e->next ? FALSE : TRUE;
+          ctk_effect_paint (e->data, paint, last_effect);
+        }
+
+      ctk_actor_set_effects_painting (ctk_actor, FALSE);
+    }
+  else
+    {
+      CLUTTER_ACTOR_CLASS (ctk_image_parent_class)->paint (actor);
+
+      clutter_actor_paint (CTK_IMAGE (actor)->priv->texture);
+    }
+  ctk_actor_set_damaged (ctk_actor, FALSE);
 }
 
 static void
@@ -241,7 +269,9 @@ pick (ClutterActor *actor, const ClutterColor *color)
 {
   CLUTTER_ACTOR_CLASS (ctk_image_parent_class)->pick (actor, color);
 
+  ctk_actor_set_effects_painting (CTK_ACTOR (actor), TRUE);
   clutter_actor_paint (CTK_IMAGE (actor)->priv->texture);
+  ctk_actor_set_effects_painting (CTK_ACTOR (actor), FALSE);
 }
 
 static void
@@ -446,11 +476,105 @@ refresh_icon (CtkImage *image)
     }
   else if (priv->type == CTK_IMAGE_GICON)
     {
-      g_debug ("CtkImageType == CTK_IMAGE_GICON not implemented");
+      if (G_IS_THEMED_ICON (priv->gicon))
+        {
+          GtkIconInfo * info;
+
+          info = gtk_icon_theme_lookup_by_gicon (priv->icon_theme,
+                                                 priv->gicon,
+                                                 priv->size,
+                                                 0);
+          if (info != NULL)
+            {
+              GdkPixbuf *pixbuf = gtk_icon_info_load_icon (info, NULL);
+              if (pixbuf)
+                {
+                  gtk_clutter_texture_set_from_pixbuf (CLUTTER_TEXTURE (priv->texture),
+                                                       pixbuf,
+                                                       NULL);
+                  priv->real_width = gdk_pixbuf_get_width (pixbuf);
+                  priv->real_height = gdk_pixbuf_get_height (pixbuf);
+
+                  g_object_unref (pixbuf);
+                }
+              else
+                {
+                  g_warning ("Unable to load gicon");
+                }
+            }
+          else
+            {
+              g_warning ("Unable to lookup gicon");
+            }
+        }
+      else if (G_IS_FILE_ICON (priv->gicon))
+        {
+          GFile *file = g_file_icon_get_file (G_FILE_ICON (priv->gicon));
+          if (G_IS_FILE (file))
+            {
+              GdkPixbuf *pixbuf;
+              GError    *error = NULL;
+
+              pixbuf = gdk_pixbuf_new_from_file_at_size (g_file_get_path (file),
+                     priv->size,
+                     priv->size,
+                     &error);
+
+              if (error)
+                {
+                  g_warning ("Unable to load icon %s: %s",
+                             g_file_get_path (file),
+                       error->message);
+                  g_error_free (error);
+                }
+
+              if (pixbuf)
+                {
+                  gtk_clutter_texture_set_from_pixbuf (CLUTTER_TEXTURE (priv->texture),
+                               pixbuf,
+                               NULL);
+
+                  priv->real_width = gdk_pixbuf_get_width (pixbuf);
+                  priv->real_height = gdk_pixbuf_get_height (pixbuf);
+
+                  g_object_unref (pixbuf);
+                }             
+            }
+        }
+      else
+        {
+          g_warning ("GIcon '%s' is not supported", g_icon_to_string (priv->gicon));
+        }
     }
   else if (priv->type == CTK_IMAGE_FILENAME)
     {
-      g_debug ("CtkImageType == CTK_IMAGE_FILENAME not implemented");
+      GdkPixbuf *pixbuf;
+      GError    *error = NULL;
+
+      pixbuf = gdk_pixbuf_new_from_file_at_size (priv->filename,
+						 priv->size,
+						 priv->size,
+						 &error);
+
+      if (error)
+	{
+	  g_warning ("Unable to load icon %s: %s",
+		     priv->filename,
+		     error->message);
+	  g_error_free (error);
+	}
+
+      if (pixbuf)
+	{
+	  gtk_clutter_texture_set_from_pixbuf (CLUTTER_TEXTURE (priv->texture),
+					       pixbuf,
+					       NULL);
+
+	  priv->real_width = gdk_pixbuf_get_width (pixbuf);
+	  priv->real_height = gdk_pixbuf_get_height (pixbuf);
+
+	  g_object_unref (pixbuf);
+	}
     }
   else
     {
@@ -470,8 +594,8 @@ refresh_icon (CtkImage *image)
 
 /**
  * ctk_image_new_from_pixbuf:
- * @size: the size of the image in pixels 
- * @pixbuf: a #GdkPixbuf object 
+ * @size: the size of the image in pixels
+ * @pixbuf: a #GdkPixbuf object
  *
  * Creates a new #CtkImage object based on the data in @pixbuf
  *
@@ -492,12 +616,12 @@ ctk_image_new_from_pixbuf (guint        size,
 
 /**
  * ctk_image_new_from_stock:
- * @size: the size of the image in pixels 
+ * @size: the size of the image in pixels
  * @stock_id: a string containing the stock id name
  *
  * Creates a new #CtkImage based on the @stock_id provided, see #GtkStockItem
  *
- * Returns: a new #CtkImage  
+ * Returns: a new #CtkImage
  */
 ClutterActor *
 ctk_image_new_from_stock (guint        size,
@@ -514,12 +638,12 @@ ctk_image_new_from_stock (guint        size,
 
 /**
  * ctk_image_new_from_icon_name:
- * @size: the size of the image in pixels 
- * @icon_name: a string containing the icon name 
+ * @size: the size of the image in pixels
+ * @icon_name: a string containing the icon name
  *
- * Uses a "named icon" to create a new CtkImage object 
+ * Uses a "named icon" to create a new CtkImage object
  *
- * Returns: a new #ClutterActor 
+ * Returns: a new #ClutterActor
  */
 ClutterActor *
 ctk_image_new_from_icon_name (guint        size,
@@ -536,11 +660,11 @@ ctk_image_new_from_icon_name (guint        size,
 
 /**
  * ctk_image_new_from_gicon:
- * @size: the size of the image in pixels 
+ * @size: the size of the image in pixels
  * @icon: a #GIcon object containing the icon
  *
- * Creates a new #CtkImage object based on the data in the provided #GIcon @icon 
- * 
+ * Creates a new #CtkImage object based on the data in the provided #GIcon @icon
+ *
  * Returns: a new #CtkImage
  */
 ClutterActor *
@@ -581,7 +705,7 @@ ctk_image_new_from_filename (guint        size,
 /**
  * ctk_image_get_image_storage_type:
  * @image: A CtkImage
- * 
+ *
  * Retrives the storage type used to create/set the CtkImage object
  *
  * Returns: a #CtkImageType object
@@ -598,8 +722,8 @@ ctk_image_get_image_storage_type (CtkImage *image)
  * ctk_image_set_size:
  * @image: A #CtkImage
  * @size: size in pixels of the image
- * 
- * Sets the size of @image to the given @size 
+ *
+ * Sets the size of @image to the given @size
  */
 void
 ctk_image_set_size (CtkImage *image,
@@ -617,7 +741,7 @@ ctk_image_set_size (CtkImage *image,
 /**
  * ctk_image_get_size:
  * @image: A #CtkImage object
- * 
+ *
  * Retrives the current size in pixels of @image
  *
  * Returns: a uint, 0 if @image is invalid
@@ -634,7 +758,7 @@ ctk_image_get_size (CtkImage *image)
  * ctk_image_set_from_pixbuf:
  * @image: A #CtkImage object
  * @pixbuf: A #GdkPixbuf object
- * 
+ *
  * sets @image to use @pixbuf for its image data
  * see ctk_image_new_from_pixbuf()
  */
@@ -657,8 +781,8 @@ ctk_image_set_from_pixbuf (CtkImage  *image,
  * ctk_image_set_from_stock:
  * @image: a CtkImage
  * @stock_id: a gchar referencing the stock image id
- * 
- * sets a CtkImage to display a stock icon 
+ *
+ * sets a CtkImage to display a stock icon
  * see ctk_image_new_from_stock()
  */
 void
@@ -680,7 +804,7 @@ ctk_image_set_from_stock (CtkImage    *image,
  * ctk_image_set_from_icon_name:
  * @image: A #CtkImage
  * @icon_name: a string representation of the icon name
- * 
+ *
  * Sets @image to use the image data supplied with the given @icon_name
  * see: ctk_image_new_from_icon_name();
  */
@@ -703,7 +827,7 @@ ctk_image_set_from_icon_name (CtkImage    *image,
  * ctk_image_set_from_gicon:
  * @image: a #CtkImage
  * @icon: A #GIcon object
- * 
+ *
  * Sets @image to use @icon as its image data
  * see: ctk_image_new_from_gicon()
  */
@@ -726,7 +850,7 @@ ctk_image_set_from_gicon (CtkImage    *image,
  * ctk_image_set_from_filename:
  * @image: a #CtkImage
  * @filename: a string representation of the filename
- * 
+ *
  * Sets @image to use @filename as its iamge data
  */
 void
