@@ -43,6 +43,8 @@
 #include "ctk-private.h"
 #include "ctk-types.h"
 
+#define SCROLL_STEP 42
+
 G_DEFINE_TYPE (CtkScrollView,
                ctk_scroll_view,
                CTK_TYPE_BIN);
@@ -65,6 +67,7 @@ struct _CtkScrollViewPrivate
 
   gfloat last_y;
   gfloat last_offset;
+  gboolean show_bars;
 };
 
 enum
@@ -158,15 +161,37 @@ ctk_scroll_view_get_property (GObject    *object,
 static void
 ctk_scroll_view_paint (ClutterActor *actor)
 {
-  CtkScrollViewPrivate *priv = CTK_SCROLL_VIEW (actor)->priv;
+  CtkActor *ctk_actor = CTK_ACTOR (actor);
+  GSList   *effects   = NULL;
 
-  CLUTTER_ACTOR_CLASS (ctk_scroll_view_parent_class)->paint (actor);
+  if (!ctk_actor_get_effects_painting (ctk_actor)
+      && (effects = ctk_actor_get_effects (ctk_actor)))
+    {
+      GSList *e;
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->trough))
-    clutter_actor_paint (priv->trough);
+      ctk_actor_set_effects_painting (ctk_actor, TRUE);
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (priv->slider))
-    clutter_actor_paint (priv->slider);
+      for (e = effects; e; e = e->next)
+        {
+          gboolean last_effect = e->next ? FALSE : TRUE;
+          ctk_effect_paint (e->data, ctk_scroll_view_paint, last_effect);
+        }
+
+      ctk_actor_set_effects_painting (ctk_actor, FALSE);
+    }
+  else
+    {
+      CtkScrollViewPrivate *priv = CTK_SCROLL_VIEW (actor)->priv;
+
+      CLUTTER_ACTOR_CLASS (ctk_scroll_view_parent_class)->paint (actor);
+
+      if (CLUTTER_ACTOR_IS_VISIBLE (priv->trough))
+        clutter_actor_paint (priv->trough);
+
+      if (CLUTTER_ACTOR_IS_VISIBLE (priv->slider))
+        clutter_actor_paint (priv->slider);
+    }
+  ctk_actor_set_damaged (ctk_actor, FALSE);
 }
 
 static void
@@ -200,6 +225,26 @@ ctk_scroll_view_preffered_height (ClutterActor *actor,
     clutter_actor_get_preferred_height (child, for_width,
                                         minimum_height, natural_height);
 }
+
+static gboolean
+show_hide_bars (CtkScrollView *self)
+{
+  g_return_val_if_fail (CTK_IS_SCROLL_VIEW (self), FALSE);
+
+  if (self->priv->show_bars)
+    {
+      clutter_actor_show (self->priv->trough);
+      clutter_actor_show (self->priv->slider);
+    }
+  else
+    {
+      clutter_actor_hide (self->priv->trough);
+      clutter_actor_hide (self->priv->slider);
+    }
+
+  return FALSE;
+}
+
 static void
 ctk_scroll_view_allocate (ClutterActor          *actor,
                           const ClutterActorBox *box,
@@ -238,10 +283,10 @@ ctk_scroll_view_allocate (ClutterActor          *actor,
     visible_scollbar = TRUE;
 
   /* Allocate the child */
-  child_box.x1 = padding.left;
-  child_box.x2 = child_box.x1 + width;
-  child_box.y1 = padding.top;
-  child_box.y2 = child_box.y1 + natural_height;
+  child_box.x1 = floor (padding.left);
+  child_box.x2 = floor (child_box.x1 + width);
+  child_box.y1 = floor (padding.top);
+  child_box.y2 = floor (child_box.y1 + natural_height);
 
   if (visible_scollbar)
     {
@@ -252,16 +297,16 @@ ctk_scroll_view_allocate (ClutterActor          *actor,
 
       if (direction == GTK_TEXT_DIR_LTR)
         {
-          child_box.x2 -= trough_width;
+          child_box.x2 -= floor (trough_width);
         }
       else
         {
-          child_box.x1 += trough_width;
-          child_box.x2 = child_box.x1 + width - trough_width;
+          child_box.x1 += floor (trough_width);
+          child_box.x2 = floor (child_box.x1 + width - trough_width);
         }
 
-      child_box.y1 -= (priv->offset/(height - slider_size)) * extra;
-      child_box.y2 = child_box.y1 + natural_height;
+      child_box.y1 -= floor ((priv->offset/(height - slider_size)) * extra);
+      child_box.y2 = floor (child_box.y1 + natural_height);
     }
 
   clutter_actor_allocate (child, &child_box, flags);
@@ -276,46 +321,47 @@ ctk_scroll_view_allocate (ClutterActor          *actor,
     {
       if (direction == GTK_TEXT_DIR_LTR)
         {
-          child_box.x1 = padding.left+width-clutter_actor_get_width (priv->trough);
-          child_box.x2 = padding.left + width;
+          child_box.x1 = floor (padding.left+width-clutter_actor_get_width (priv->trough));
+          child_box.x2 = floor (padding.left + width);
         }
       else
         {
-          child_box.x1 = padding.left;
-          child_box.x2 = padding.left + clutter_actor_get_width (priv->trough);
+          child_box.x1 = floor (padding.left);
+          child_box.x2 = floor (padding.left + clutter_actor_get_width (priv->trough));
         }
-      child_box.y1 = padding.top;
-      child_box.y2 = padding.top + height;
+      child_box.y1 = floor (padding.top);
+      child_box.y2 = floor (padding.top + height);
 
       clutter_actor_allocate (priv->trough, &child_box, flags);
-      clutter_actor_show (priv->trough);
 
       /* Slider */
       if (direction == GTK_TEXT_DIR_LTR)
         {
-          child_box.x1 = padding.left+width-clutter_actor_get_width (priv->slider);
-          child_box.x2 = padding.left+width;
+          child_box.x1 = floor (padding.left+width-clutter_actor_get_width (priv->slider));
+          child_box.x2 = floor (padding.left+width);
         }
       else
         {
-          child_box.x1 = padding.left;
-          child_box.x2 = padding.left + clutter_actor_get_width (priv->slider);
+          child_box.x1 = floor (padding.left);
+          child_box.x2 = floor (padding.left + clutter_actor_get_width (priv->slider));
         }
-      child_box.y1 = padding.top + priv->offset;
-      child_box.y2 = child_box.y1 + ((height/natural_height) *height);
+      child_box.y1 = floor (padding.top + priv->offset);
+      child_box.y2 = floor (child_box.y1 + ((height/natural_height) *height));
 
       clutter_actor_allocate (priv->slider, &child_box, flags);
-      clutter_actor_show (priv->slider);
+
+      priv->show_bars = TRUE;
+      g_timeout_add (0, (GSourceFunc)show_hide_bars, actor);
     }
   else
     {
-      clutter_actor_hide (priv->slider);
-      clutter_actor_hide (priv->trough);
+      priv->show_bars = FALSE;
+      g_timeout_add (0, (GSourceFunc)show_hide_bars, actor);
     }
 
-  priv->slider_height = (height/natural_height) *height;
-  priv->child_natural_height = natural_height;
-  priv->self_height = height;
+  priv->slider_height = floor ((height/natural_height) *height);
+  priv->child_natural_height = floor (natural_height);
+  priv->self_height = floor (height);
 }
 
 static void
@@ -329,7 +375,11 @@ ctk_scroll_view_pick (ClutterActor       *actor,
    * the child actors also getting 'picked'.
    */
   if (CLUTTER_ACTOR_IS_VISIBLE (actor))
-    ctk_scroll_view_paint (actor);
+    {
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), TRUE);
+      ctk_scroll_view_paint (actor);
+      ctk_actor_set_effects_painting (CTK_ACTOR (actor), FALSE);
+    }
 }
 
 static void
@@ -449,6 +499,7 @@ ctk_scroll_view_init (CtkScrollView *self)
   priv->offset = 0;
   priv->bar_type = CTK_SCROLLBAR_INSET;
   priv->value = 0.0;
+  priv->show_bars = TRUE;
 
   clutter_actor_set_reactive (CLUTTER_ACTOR (self), TRUE);
 
@@ -565,29 +616,29 @@ on_child_focus_changed (ClutterActor    *scroll_child,
                         CtkScrollView   *self)
 {
   CtkScrollViewPrivate *priv;
-  gfloat y1, y2;
-  
+  gfloat y10, y20;
+
   g_return_if_fail (CTK_IS_SCROLL_VIEW (self));
   priv = self->priv;
 
   if (!focus_child_box)
     return;
 
-  y1 = priv->offset;
-  y2 = priv->offset + priv->self_height;
+  y10 = priv->offset;
+  y20 = priv->offset + priv->self_height;
 
-  if (focus_child_box->y1 < y1)
+  if (focus_child_box->y1 < y10)
     {
       gfloat diff;
-      
-      diff = y1;
+
+      diff = y10;
       diff = diff / priv->child_natural_height;
       ctk_scroll_view_set_value (self, diff);
     }
-  else if (focus_child_box->y2 > y2)
+  else if (focus_child_box->y2 > y20)
     {
       gfloat diff;
-      
+
       diff = focus_child_box->y2;
       diff = diff / priv->child_natural_height;
       ctk_scroll_view_set_value (self, diff);
@@ -639,7 +690,7 @@ ctk_scroll_view_scroll (CtkScrollView *view,
   if (priv->child_natural_height <= priv->self_height)
     return FALSE;
 
-  diff = (priv->child_natural_height - priv->self_height) * 0.050;
+  diff = SCROLL_STEP;
 
   priv->offset += (event->direction == CLUTTER_SCROLL_UP) ? -(diff) : diff;
 
@@ -656,7 +707,7 @@ ctk_scroll_view_scroll (CtkScrollView *view,
 
 /**
  * ctk_scroll_view_set_bar_type:
- * @view: a #CtkScrollView object 
+ * @view: a #CtkScrollView object
  * @type: a #CtkScrollbarType object
  *
  * sets @view to use the type of scrollbar defined in @type. see: #CtkScrollbarType
@@ -684,7 +735,7 @@ ctk_scroll_view_set_bar_type (CtkScrollView      *view,
 /**
  * ctk_scroll_view_get_bar_type:
  * @view: a #CtkScrollView object
- * 
+ *
  * Retrives the scrollbar type of @view
  *
  * Returns: a #CtkScrollbarType
@@ -754,7 +805,7 @@ ctk_scroll_view_set_value (CtkScrollView *view,
 /**
  * ctk_scroll_view_get_value:
  * @view: A #CtkScrollView
- * 
+ *
  * Retrives the current scroll position (from 0.0 -> 1.0) of @view
  *
  * Returns: a float from 0.0 to 1.0
@@ -776,7 +827,7 @@ ctk_scroll_view_get_value (CtkScrollView *view)
  * @trough: a #ClutterActor object
  * @slider: a #ClutterActor object
  *
- * Sets @view to use @trough and @slider as its actors for its scrollbar 
+ * Sets @view to use @trough and @slider as its actors for its scrollbar
  */
 void
 ctk_scroll_view_set_scroll_bar (CtkScrollView      *view,
@@ -827,7 +878,7 @@ ctk_scroll_view_set_scroll_bar (CtkScrollView      *view,
     clutter_actor_queue_relayout (CLUTTER_ACTOR (view));
 }
 
-/** 
+/**
  * ctk_scroll_view_get_scroll_bar:
  * @view: a #CtkScrollView object
  * @trough: a Pointer to a #ClutterActor object
@@ -856,7 +907,7 @@ ctk_scroll_view_get_scroll_bar (CtkScrollView      *view,
  * @view: a #CtkScrollView object
  *
  * Retrives wheather @view can be scrolled or not
- * 
+ *
  * Returns: a boolean value, false if @view is invalid
  */
 gboolean
